@@ -1,17 +1,16 @@
 ;;; calc-keypd.el --- mouse-capable keypad input for Calc
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2013 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 ;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -299,8 +296,7 @@
 	      (setq win (split-window win (+ width 7) t))
 	      (set-window-buffer win calcbuf))
 	  (if (or t  ; left-side keypad not yet fully implemented
-		  (< (save-excursion
-		       (set-buffer (window-buffer old-win))
+		  (< (with-current-buffer (window-buffer old-win)
 		       (current-column))
 		     (/ (window-width) 2)))
 	      (setq win (split-window old-win (- (window-width old-win)
@@ -384,7 +380,7 @@
 	  (delete-region (point-min) (point))
 	  (if calc-keypad-input
 	      (insert "Calc: " calc-keypad-input "\n")
-	    (insert "----+-----Calc " calc-version " -----+----"
+	    (insert "----+----+--Calc---+----+----"
 		    (int-to-string (1+ calc-keypad-menu))
 		    "\n")))))
   (setq calc-keypad-prev-input calc-keypad-input))
@@ -393,9 +389,7 @@
   (interactive)
   (unless (eq major-mode 'calc-keypad-mode)
     (error "Must be in *Calc Keypad* buffer for this command"))
-  (let* ((row (save-excursion
-		(beginning-of-line)
-		(count-lines (point-min) (point))))
+  (let* ((row (count-lines (point-min) (point-at-bol)))
 	 (y (/ row 2))
 	 (x (/ (current-column) (if (>= y 4) 6 5)))
 	 radix frac inv
@@ -549,8 +543,7 @@
 (defun calc-keypad-right-click (event)
   "Handle a right-button mouse click in Calc Keypad window."
   (interactive "e")
-  (save-excursion
-    (set-buffer calc-keypad-buffer)
+  (with-current-buffer calc-keypad-buffer
     (calc-keypad-menu)))
 
 (defun calc-keypad-middle-click (event)
@@ -610,7 +603,7 @@
 				   negative-argument))
 		       (and prefix (string-match "\\`\e?[-0-9]\\'" keys)))
 		   (progn
-		     (setq last-command-char (aref keys (1- (length keys))))
+		     (setq last-command-event (aref keys (1- (length keys))))
 		     (command-execute cmd)
 		     (setq flush 'not-any-more
 			   prefix t
@@ -623,5 +616,4 @@
 
 (provide 'calc-keypd)
 
-;;; arch-tag: 4ba0d360-2bb6-40b8-adfa-eb373765b3f9
 ;;; calc-keypd.el ends here
