@@ -1,9 +1,9 @@
 ;;; erc-lang.el --- provide the LANG command to ERC
 
-;; Copyright (C) 2002, 2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 2002, 2004, 2006-2013 Free Software Foundation, Inc.
 
 ;; Author: Alex Schroeder <alex@gnu.org>
-;; Maintainer: Alex Schroeder <alex@gnu.org>
+;; Maintainer: FSF
 ;; Version: 1.0.0
 ;; URL: http://www.emacswiki.org/cgi-bin/wiki.pl?ErcLang
 ;; Keywords: comm languages processes
@@ -202,10 +202,9 @@ Normungsinstitut (ON), Postfach 130, A-1021 Vienna, Austria.")
   (let ((lang (cdr (assoc language iso-638-languages))))
     (erc-display-message
      nil 'notice 'active
-     (or lang (concat line ": No such domain"))))
+     (or lang (concat language ": No such domain"))))
   t)
 
 (provide 'erc-lang)
 
-;; arch-tag: 8ffb1563-cc03-4517-b067-16309d4ff97b
 ;;; erc-lang.el ends here
