@@ -1,7 +1,6 @@
 ;;; nnvirtual.el --- virtual newsgroups access for Gnus
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-;;   2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 1994-2013 Free Software Foundation, Inc.
 
 ;; Author: David Moore <dmoore@ucsd.edu>
 ;;	Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -93,8 +92,7 @@ component group will show up when you enter the virtual group.")
 (deffoo nnvirtual-retrieve-headers (articles &optional newsgroup
 					     server fetch-old)
   (when (nnvirtual-possibly-change-server server)
-    (save-excursion
-      (set-buffer nntp-server-buffer)
+    (with-current-buffer nntp-server-buffer
       (erase-buffer)
       (if (stringp (car articles))
 	  'headers
@@ -170,8 +168,7 @@ component group will show up when you enter the virtual group.")
 	  ;; the nntp-server-buffer, which is where Gnus expects to find
 	  ;; them.
 	  (prog1
-	      (save-excursion
-		(set-buffer nntp-server-buffer)
+	      (with-current-buffer nntp-server-buffer
 		(erase-buffer)
 		(insert-buffer-substring vbuf)
 		;; FIX FIX FIX, we should be able to sort faster than
@@ -197,10 +194,11 @@ component group will show up when you enter the virtual group.")
 	    (when buffer
 	      (set-buffer buffer))
 	    (let* ((gnus-override-method nil)
-		   (method (gnus-find-method-for-group
-			    nnvirtual-last-accessed-component-group)))
-	      (funcall (gnus-get-function method 'request-article)
-		       article nil (nth 1 method) buffer)))))
+		   (gnus-command-method
+		    (gnus-find-method-for-group
+		     nnvirtual-last-accessed-component-group)))
+	      (funcall (gnus-get-function gnus-command-method 'request-article)
+		       article nil (nth 1 gnus-command-method) buffer)))))
       ;; This is a fetch by number.
       (let* ((amap (nnvirtual-map-article article))
 	     (cgroup (car amap)))
@@ -215,8 +213,7 @@ component group will show up when you enter the virtual group.")
 	 (t
 	  (setq nnvirtual-last-accessed-component-group cgroup)
 	  (if buffer
-	      (save-excursion
-		(set-buffer buffer)
+	      (with-current-buffer buffer
 		;; We bind this here to avoid double decoding.
 		(let ((gnus-article-decode-hook nil))
 		  (gnus-request-article-this-buffer (cdr amap) cgroup)))
@@ -250,7 +247,7 @@ component group will show up when you enter the virtual group.")
       t)))
 
 
-(deffoo nnvirtual-request-group (group &optional server dont-check)
+(deffoo nnvirtual-request-group (group &optional server dont-check info)
   (nnvirtual-possibly-change-server server)
   (setq nnvirtual-component-groups
 	(delete (nnvirtual-current-group) nnvirtual-component-groups))
@@ -260,13 +257,11 @@ component group will show up when you enter the virtual group.")
     (nnheader-report 'nnvirtual "No component groups in %s" group))
    (t
     (setq nnvirtual-current-group group)
-    (when (or (not dont-check)
-	      nnvirtual-always-rescan)
-      (nnvirtual-create-mapping)
-      (when nnvirtual-always-rescan
-	(nnvirtual-request-update-info
-	 (nnvirtual-current-group)
-	 (gnus-get-info (nnvirtual-current-group)))))
+    (nnvirtual-create-mapping dont-check)
+    (when nnvirtual-always-rescan
+      (nnvirtual-request-update-info
+       (nnvirtual-current-group)
+       (gnus-get-info (nnvirtual-current-group))))
     (nnheader-insert "211 %d 1 %d %s\n"
 		     nnvirtual-mapping-len nnvirtual-mapping-len group))))
 
@@ -300,10 +295,6 @@ component group will show up when you enter the virtual group.")
   t)
 
 
-(deffoo nnvirtual-request-list (&optional server)
-  (nnheader-report 'nnvirtual "LIST is not implemented."))
-
-
 (deffoo nnvirtual-request-newgroups (date &optional server)
   (nnheader-report 'nnvirtual "NEWGROUPS is not supported."))
 
@@ -341,8 +332,7 @@ component group will show up when you enter the virtual group.")
 	 (when (not (numberp (gnus-group-unread g)))
 	   (gnus-activate-group g)))
        nnvirtual-component-groups)
-      (save-excursion
-	(set-buffer gnus-group-buffer)
+      (with-current-buffer gnus-group-buffer
 	(gnus-group-catchup-current nil all)))))
 
 
@@ -674,7 +664,7 @@ the result."
     carticles))
 
 
-(defun nnvirtual-create-mapping ()
+(defun nnvirtual-create-mapping (dont-check)
   "Build the tables necessary to map between component (group, article) to virtual article.
 Generate the set of read messages and marks for the virtual group
 based on the marks on the component groups."
@@ -693,7 +683,9 @@ based on the marks on the component groups."
     ;; Into all-marks we put (g marks).
     ;; We also increment cnt and tot here, and compute M (max of sizes).
     (mapc (lambda (g)
-	    (setq active (gnus-activate-group g)
+	    (setq active (or (and dont-check
+				  (gnus-active g))
+			     (gnus-activate-group g))
 		  min (car active)
 		  max (cdr active))
 	    (when (and active (>= max min) (not (zerop max)))
@@ -809,5 +801,4 @@ based on the marks on the component groups."
 
 (provide 'nnvirtual)
 
-;; arch-tag: ca8c8ad9-1bd8-4b0f-9722-90dc645a45f5
 ;;; nnvirtual.el ends here
