@@ -1,8 +1,8 @@
 ;;; korea-util.el --- utilities for Korean
 
-;; Copyright (C) 1997, 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007
-;;   Free Software Foundation, Inc.
-;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+;; Copyright (C) 1997, 1999, 2001-2013 Free Software Foundation, Inc.
+;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+;;   2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
 ;;   Registration Number H14PRO021
 
@@ -10,10 +10,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,9 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -31,10 +29,10 @@
 
 ;;;###autoload
 (defvar default-korean-keyboard
-  (if (string-match "3" (or (getenv "HANGUL_KEYBOARD_TYPE") ""))
+   (purecopy (if (string-match "3" (or (getenv "HANGUL_KEYBOARD_TYPE") ""))
       "3"
-    "")
-  "*The kind of Korean keyboard for Korean input method.
+    ""))
+   "The kind of Korean keyboard for Korean input method.
 \"\" for 2, \"3\" for 3.")
 
 ;; functions useful for Korean text input
@@ -43,7 +41,7 @@
   "Turn on or off a Korean text input method for the current buffer."
   (interactive)
   (if current-input-method
-      (inactivate-input-method)
+      (deactivate-input-method)
     (activate-input-method
      (concat "korean-hangul" default-korean-keyboard))))
 
@@ -98,14 +96,18 @@
 ;; Information for setting and exiting Korean environment.
 (defvar korean-key-bindings
   `((global [?\S- ] toggle-korean-input-method nil)
+    (global [Hangul] toggle-korean-input-method nil)
     (global [C-f9] quail-hangul-switch-symbol-ksc nil)
-    (global [f9]  quail-hangul-switch-hanja nil)
+    (global [f9] hangul-to-hanja-conversion nil)
+    (global [Hangul_Hanja] hangul-to-hanja-conversion nil)
     (,isearch-mode-map [?\S- ] isearch-toggle-korean-input-method nil)
+    (,isearch-mode-map [Hangul] isearch-toggle-korean-input-method nil)
     (,isearch-mode-map [C-f9] isearch-hangul-switch-symbol-ksc nil)
     (,isearch-mode-map [f9] isearch-hangul-switch-hanja nil)))
 
 ;;;###autoload
 (defun setup-korean-environment-internal ()
+  (use-cjk-char-width-table 'ko_KR)
   (let ((key-bindings korean-key-bindings))
     (while key-bindings
       (let* ((this (car key-bindings))
@@ -123,6 +125,7 @@
 
 (defun exit-korean-environment ()
   "Exit Korean language environment."
+  (use-default-char-width-table)
   (let ((key-bindings korean-key-bindings))
     (while key-bindings
       (let* ((this (car key-bindings))
@@ -139,5 +142,4 @@
 ;;
 (provide 'korea-util)
 
-;;; arch-tag: b17d0981-05da-4577-99f8-1db87fff8b44
 ;;; korea-util.el ends here
