@@ -1,17 +1,16 @@
 ;;; rfc2368.el --- support for rfc2368
 
-;; Copyright (C) 1998, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;; Copyright (C) 1998, 2000-2013 Free Software Foundation, Inc.
 
 ;; Author: Sen Nagata <sen@eccosys.com>
 ;; Keywords: mail
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 ;;
@@ -36,7 +33,7 @@
 ;;
 ;;   this is intended as a replacement for mailto.el
 ;;
-;; acknowledgements:
+;; acknowledgments:
 ;;
 ;;   the functions that deal w/ unhexifying in this file were basically
 ;; taken from w3 -- i hope to replace them w/ something else soon OR
@@ -94,13 +91,11 @@ Note: make sure MAILTO-URL has been 'unhtmlized' (e.g. &amp; -> &), before
 calling this function."
   (let ((case-fold-search t)
 	prequery query headers-alist)
-
+    (setq mailto-url (replace-regexp-in-string "\n" " " mailto-url))
     (if (string-match rfc2368-mailto-regexp mailto-url)
 	(progn
-
 	  (setq prequery
 		(match-string rfc2368-mailto-prequery-index mailto-url))
-
 	  (setq query
 		(match-string rfc2368-mailto-query-index mailto-url))
 
@@ -133,10 +128,8 @@ calling this function."
 
 	  headers-alist)
 
-      (error "Failed to match a mailto: url"))
-    ))
+      (error "Failed to match a mailto: url"))))
 
 (provide 'rfc2368)
 
-;;; arch-tag: ea804934-ad96-4f69-957b-857a76e4fd95
 ;;; rfc2368.el ends here
